#include <os.h>
#include "screen.h"
#include "charmap.h"

inline void locate(char row, char col, char* str, char bgColor, char textColor)
{
	putStr(col*CHAR_WIDTH,row*CHAR_HEIGHT,str,bgColor,textColor);
}
inline void locatec(char row, char col, char ch, char bgColor, char textColor)
{
	putChar(col*CHAR_WIDTH,row*CHAR_HEIGHT,ch,bgColor,textColor);
}

struct console
{
	char buffer[64][40];
	// both colors could be together in one byte
	char bgColor[64][40];
	char textColor[64][40];
	int cursorx;
	int cursory;
};

void initConsole(struct console *console, char defaultBgColor, char defaultTextColor)
{
	memset(console->buffer,0,sizeof(console->buffer));
	memset(console->bgColor,defaultBgColor,sizeof(console->bgColor));
	memset(console->textColor,defaultTextColor,sizeof(console->textColor));
	console->cursorx = 0;
	console->cursory = 0;
}
void drawConsole(struct console *c)
{
	int row,col;
	for(row = 0; row < 40; row++)
	{
		for(col = 0; col < 64; col++)
		{
			if(c->buffer[col][row] != '\0')
			{
				locatec(row,col,c->buffer[col][row],c->bgColor[col][row],c->textColor[col][row]);
			}
			else
			{
				locatec(row,col,' ',c->bgColor[col][row],c->textColor[col][row]);
			}
		}
	}
}
void c_write(struct console *c, char* str, char bgColor, char textColor)
{
	int len = strlen(str);
	int i;
	for(i = 0; i < len; i++)
	{
		if(str[i] == '\n')
		{
			if(c->cursory >= 40)
			{
				// Console should scroll one line down here
				// but for now, just do nothing
			}
			else
			{
				c->cursory++;
				c->cursorx = 0;
			}
		}
		else if(str[i] == '\r')
		{
			c->cursorx = 0;
		}
		else if(str[i] == '\b')
		{
			if(c->cursorx > 0)
			{
				c->cursorx--;
			}
		}
		else
		{
			if(c->cursorx >= 64)
			{
				if(c->cursory >= 40)
				{
					// Should scroll down here too
				}
				else
				{
					c->cursory++;
					c->cursorx = 0;
				}
			}
			c->buffer[c->cursorx][c->cursory] = str[i];
			c->bgColor[c->cursorx][c->cursory] = bgColor;
			c->textColor[c->cursorx][c->cursory] = textColor;
			c->cursorx++;
		}
	}
}
void c_swrite(struct console *c, char* format, int len, char bgColor, char textColor, ...)
{
	char buf[len];
	memset(buf,'\0',sizeof(char)*len);
	va_list arglist;
	va_start(arglist,textColor);
	vsprintf(buf,format,arglist);
	c_write(c,buf,bgColor,textColor);
	va_end(arglist);
}